function [f] = cell_dCdT(t,c,C_T,P1,x_AtC);

% FUNCTION CELL_DCDT: computes the time derivatives for a cardiac cell 
% model of oxygen transport and oxidative phosphorylation.
% A detailed description is given in: 
%   Beard, DA. integrated computational modeling of oxygen transport and 
%   cellular energetics explains observations on in vivo cardiac oxygen 
%   consumption and energy metabolites. PLoS Comput Biol (in press), 2006.
%
% INPUTS: 
%   t - time (not used)
%   c - vector of state variables
%   C_T,P1  table of total oxygen concentration and pO2 in blood
%   x_AtC   ATP consumption parameter
%
% OUTPUTS:
%   f - vector of time derivatives of concentrations

% Setting parameters values
RT = 2.5775;                    % kJ  mol^{-1}
F = 0.096484;                   % kJ mol^{-1} mV^{-1}
dG_C1o = -69.37;                % kJ mol^{-1}
dG_C3o = -32.53;                % kJ mol^{-1}
dG_C4o = -122.94;               % kJ mol^{-1}
dG_F1o = 36.03;                 % kJ mol^{-1}
n_A = 3.0;                      % unitless
pH_c = 7.1;                     % cytosol pH
H_c = 10^(-pH_c);               % Molar
K_c = 150.0e-3;                 % Molar
k_dHPi  = 10^(-6.75);           % H/Pi binding constant Molar
k_dHatp = 10^(-6.48);           % H/ATP binding constant Molar
k_dHadp = 10^(-6.29);           % H/ADP binding constant Molar
K_DT = 24e-6;                   % Molar
K_DD = 347e-6;                  % Molar
K_AK = 0.4331;                  % nondimensional
K_CK = 1.66e9;                  % ref from Balaban paper
W_m = 0.664*1.09;               % mitochondrial water space (ml water per ml mito) [Vinnakota & Bassingthwaighte]
W_x = 0.9*W_m;                  % Matrix water space
W_i = 0.1*W_m;                  % IM water space
Rm_cyto = 0.20 / 0.472;         % Volume ratio mito volume / cytoplasm volume
Rm_cell = 0.20 / 0.694;         % Volume ratio mito volume / cell volume
Rc_cell = 0.472 / 0.694;        % Volume ratio of cytosol / cell volume
W_c = 0.807*1.044;              % cytosol water space (ml water per ml cytosol) [Vinnakota & Bassingthwaighte]
V1 = 0.05;                      % unitless
V2 = 0.17585;                   % unitless
V3 = 0.73078;                   % unitless
gamma  = 5.99;                  % mito membrane area per cell volume micron^{-1}
Ctot   = 2.70e-3;               % M; total cytoC
Qtot   = 1.35e-3;               % M; total Q+QH2
NADtot = 2.97e-3;               % M; total NAD+NADH
CRtot  = (27.3e-3)*(0.694/0.472);% Total Cr+PCr
a_1 = 1.30e-6;                  % oxygen solubility in cap
a_2 = 1.25e-6;                  % oxygen solubility in isf
a_3 = 1.74e-6;                  % oxygen solubility in cell
CMb = 200e-6;                   % Myoglobin concentration (moles per liter cell volume)
C50 = a_3*(2.39);               % oxy-myoglobin 1/2 saturation
PS12 = 50;                      % PS product--capillary/ISF
PS23 = 10;                      % PS product--ISF/cell

% OxPhos parameters:
k_Pi1  = 0.1553e-3;
k_Pi2  = 0.8222e-3;
k_Pi3  = 0.3601e-3;
k_Pi4  = 5.651e-3;
k_PiH  = 0.2542e-3;
r      = 4.559;
x_DH   = 0.0866;
x_C1   = 4405; 
x_C3   = 4.887;
x_C4   = 6.766e-5;
x_F1   = 1000.0;
x_ANT  = 0.008123;
x_Pi1  = 3.850e5; 
x_KH   = 5.651e7;
x_Hle  = 200.0;
x_K    = 0;

% Other parameters:
k_O2   = 1.2e-4;
k_mADP = 3.5e-6;
x_buff = 100;
x_AK   = 1e7;
x_CK   = 1e7;
x_MgA  = 1e7;
x_A    = 85;        % micron sec^{-1}
x_Pi2  = 327;       % micron sec^{-1}

% Concentration state variables:
c(1:30) = max(c(1:30),0);
% Oxygen concentrations:
CtO1 = c(1);
CtO2 = c(2);
CtO3 = c(3);
% Other state varialbes:
pH_x   = c(4); H_x = 10^(-pH_x);
K_x    = c(5);
Mg_x   = c(6);
NADH_x = c(7);
Q      = c(8);
Cox    = c(9);
ATP_x  = c(10);
ADP_x  = c(11);
ATP_mx = c(12);
ADP_mx = c(13);
Pi_x   = c(14);
ATP_i  = c(15);
ADP_i  = c(16);
AMP_i  = c(17);
ATP_mi = c(18);
ADP_mi = c(19);
Pi_i   = c(20);
Mg_i   = c(21);
ATP_c  = c(22);
ADP_c  = c(23);
AMP_c  = c(24);
ATP_mc = c(25);
ADP_mc = c(26);
Pi_c   = c(27);
Mg_c   = c(28);
PCr_c  = c(29);
dPsi   = c(30);

% Other concentrations computed from the state variables:
NAD_x  = NADtot - NADH_x;
QH2    = Qtot - Q;
Cred   = Ctot - Cox;
ATP_fx = ATP_x - ATP_mx;
ADP_fx = ADP_x - ADP_mx;
ATP_fi = ATP_i - ATP_mi;
ADP_fi = ADP_i - ADP_mi;
ATP_fc = ATP_c - ATP_mc;
ADP_fc = ADP_c - ADP_mc;
Cr_c   = CRtot - PCr_c;

% Computing free oxygen concentrations from total oxygen concentrations
CfO1 = a_1.*interp1(C_T,P1,CtO1,'linear');
CfO2 = CtO2;
CfO3 = ( CtO3-(C50+CMb) + sqrt( (CtO3-(C50+CMb)).^2 + 4*C50.*CtO3 ) )/2;

% K+, H+ in IM space
H_i  = H_c;
K_i  = K_c;

% Membrane proton motive force:
dG_H = F*dPsi + RT*log(H_i/H_x);

dG_C1op = dG_C1o - 1*RT*log(H_x/1e-7);
dG_C3op = dG_C3o + 2*RT*log(H_x/1e-7);
dG_C4op = dG_C4o - 2*RT*log(H_x/1e-7);

% Fluxes:
J_DH = x_DH*(r*NAD_x-NADH_x)*((1+Pi_x/k_Pi1)/(1+Pi_x/k_Pi2));
J_C1 = x_C1*( exp(-(dG_C1op+4*dG_H)/RT)*NADH_x*Q - NAD_x*QH2 );
J_C3 = x_C3*((1+Pi_x/k_Pi3)/(1+Pi_x/k_Pi4))*(exp(-(dG_C3op+4*dG_H-2*F*dPsi)/(2*RT))*Cox*sqrt(QH2) - Cred*sqrt(Q) );
J_C4 = x_C4*(CfO3/(CfO3+k_O2))*(Cred/Ctot)*( exp(-(dG_C4op+2*dG_H)/(2*RT))*Cred*(CfO3^0.25) - Cox*exp(F*dPsi/RT) );
J_F1   = x_F1*( exp(-(dG_F1o-n_A*dG_H)/RT)*(K_DD/K_DT)*ADP_mx*Pi_x - ATP_mx );
Psi_x = -0.65*dPsi;
Psi_i = +0.35*dPsi;
J_ANT = x_ANT*( ADP_fi/(ADP_fi+ATP_fi*exp(-F*Psi_i/RT)) - ADP_fx/(ADP_fx+ATP_fx*exp(-F*Psi_x/RT)) )*(ADP_fi/(ADP_fi+k_mADP));
H2PIi = Pi_i*H_i/(H_i+k_dHPi); H2PIx = Pi_x*H_x/(H_x+k_dHPi);
J_Pi1 = x_Pi1*(H_i*H2PIi - H_x*H2PIx)/(H2PIi+k_PiH);
if abs(dPsi) > 1e-3
  J_Hle = x_Hle*dPsi*(H_i*exp(F*dPsi/RT)-H_x)/(exp(F*dPsi/RT)-1);
else
  J_Hle = x_Hle*RT*(H_i-H_x)/F;
end
J_KH   = x_KH*( K_i*H_x - K_x*H_i );
if abs(dPsi) > 1e-3
  J_K   = x_K*dPsi*(K_i*exp(F*dPsi/RT)-K_x)/(exp(F*dPsi/RT)-1);
else
  J_K   = x_K*RT*(K_i-K_x)/F;
end
J_AKi  = x_AK*( K_AK*ADP_i*ADP_i - AMP_i*ATP_i );
J_AKe  = x_AK*( K_AK*ADP_c*ADP_c - AMP_c*ATP_c );
J_AMP  = gamma*x_A*(AMP_c-AMP_i);
J_ADP  = gamma*x_A*(ADP_c-ADP_i);
J_ATP  = gamma*x_A*(ATP_c-ATP_i);
J_Pi2  = gamma*x_Pi2*(Pi_c-Pi_i);    
J_MgATPx = x_MgA*(ATP_fx*Mg_x-K_DT*ATP_mx);
J_MgADPx = x_MgA*(ADP_fx*Mg_x-K_DD*ADP_mx);
J_MgATPi = x_MgA*(ATP_fi*Mg_i-K_DT*ATP_mi);
J_MgADPi = x_MgA*(ADP_fi*Mg_i-K_DD*ADP_mi);
J_MgATPe = x_MgA*(ATP_fc*Mg_c-K_DT*ATP_mc);
J_MgADPe = x_MgA*(ADP_fc*Mg_c-K_DD*ADP_mc);

% ATP consumption:
% J_AtC  = x_AtC*(ATP_c) / ( (ATP_c) + (10*ADP_c) );
J_AtC  = x_AtC;

% Creatine Kinase
J_CKe  = x_CK*( K_CK*ADP_c*PCr_c*H_c - ATP_c*Cr_c );

% Calculating permeation fluxes
% Exchange matrix:
A = [ (a_1/a_1)*(-PS12/V1)  (a_1/a_2)*(+PS12/V1)         0
      (a_1/a_1)*(+PS12/V2)  (a_1/a_2)*(-(PS12+PS23)/V2)  (a_1/a_3)*(+PS23/V2)
      0                     (a_1/a_2)*(+PS23/V3)         (a_1/a_3)*(-PS23/V3) ];
% Exchange fluxes:
J_ex  = A*[CfO1 CfO2 CfO3]';

% Time derivatives of state variables:
f(1)  = J_ex(1);
f(2)  = J_ex(2);
f(3)  = J_ex(3) + (-J_C4/2)*Rm_cell;
f(4)  = -(1/2.303*x_buff)*( +1*J_DH - (4+1)*J_C1 - (4-2)*J_C3 - (2+2)*J_C4 + (n_A-1)*J_F1 + 2*J_Pi1 + J_Hle - J_KH ) / W_x; % H_x
f(5)  = (J_KH + J_K)/W_x; % K_x
f(6)  = (-J_MgATPx - J_MgADPx)/W_x; % Mg_x
f(7)  = (+J_DH - J_C1)/W_x; % NADH
f(8)  = -(+J_C1 - J_C3)/W_x; % Q
f(9)  = -(+2*J_C3 - 2*J_C4)/W_i; % Cox
f(10) = (+J_F1 - J_ANT)/W_x; % ATP_x
f(11) = (-J_F1 + J_ANT)/W_x; % ADP_x
f(12) = (J_MgATPx)/W_x; % ATP_mx
f(13) = (J_MgADPx)/W_x; % ADP_mx
f(14) = (-J_F1 + J_Pi1)/W_x;  % Pi_x
f(15) = (+J_ATP + J_ANT + J_AKi)/W_i; % ATP_i
f(16) = (+J_ADP - J_ANT - 2*J_AKi)/W_i; % ADP_i
f(17) = (+J_AMP + J_AKi)/W_i; % AMP_i
f(18) = (J_MgATPi)/W_i; % ATP_mi
f(19) = (J_MgADPi)/W_i; % ADP_mi
f(20) = (-J_Pi1 + J_Pi2)/W_i; % Pi_i
f(21) = (-J_MgATPi - J_MgADPi)/W_i; % Mg_i
f(22) = (-J_ATP*Rm_cyto + J_AKe - J_AtC/Rc_cell + J_CKe)/W_c; % ATP_c
f(23) = (-J_ADP*Rm_cyto - 2*J_AKe + J_AtC/Rc_cell - J_CKe)/W_c; % ADP_c
f(24) = (-J_AMP*Rm_cyto + J_AKe)/W_c; % AMP_c
f(25) = (+J_MgATPe)/W_c; % ATP_mc
f(26) = (+J_MgADPe)/W_c; % ADP_mc
f(27) = (-J_Pi2*Rm_cyto + J_AtC/Rc_cell)/W_c; % Pi_c
f(28) = (-J_MgATPe - J_MgADPe)/W_c; % Mg_c
f(29) = (-J_CKe)/W_c; % PCr_c
f(30) = (1.48e5)*( 4*J_C1 + 2*J_C3 + 4*J_C4 - n_A*J_F1 - J_ANT - J_Hle - J_K ); % d_Psi
f = f';



