%%
% Step 1. Setting simulation parameters
% Oxygen solubility parameters:
a_1 = 1.30e-6;          % Moles dissolved O2 (liter blood)^{-1} mmHg^{-1}
a_2 = 1.25e-6;          % Moles dissolved O2 (liter ISF)^{-1} mmHg^{-1}
a_3 = 1.74e-6;          % Moles dissolved O2 (liter cell)^{-1} mmHg^{-1}
% Oxyhemoglobin parameters:
n = 2.55;               % unitless
P50 = 30;               % mmHg
H = 0.45;               % hematocrit unitless
CHb = 0.0213;           % red cell oxygen binding site concentration mol / (liter rbc)
Cin = a_1*100 + (CHb*H)*(100^n) / ( 100^n + P50^n );
P1 = (0:0.5:101);
C_T = a_1.*P1 + (CHb*H).*(P1.^n) ./ ( P1.^n + P50^n ); % mol / (liter blood)
P1 = [-1 P1];
C_T = [-a_1 C_T];
% Myoglobin parameters:
CMb = 200e-6;           % Myoglobin concentration (moles per liter cell volume)
C50 = a_3*(2.39);       % oxy-myoglobin 1/2 saturation
% Tissue density parameters:
rho = 1.053;            % tissue density in g per ml
V1 = 0.05;              % unitless (capillary volume fraction)
% Water space fractions:
W_m = 0.664*1.09;       % mitochondrial water space (ml water per ml mito) [Vinnakota & Bassingthwaighte]
W_x = 0.9*W_m;          % Matrix water space (ml matrix water per ml mito)
W_i = 0.1*W_m;          % IM water space (ml IM water per ml mito)
W_c = 0.807*1.044;      % cytosol water space (ml water per ml cytosol) [Vinnakota & Bassingthwaighte]
% Numerical parameters
Nseg = 60;

%%
% Step 2. Initial values of state variables:
C_i(1)  = 0.99*Cin;
C_i(2)  = 100*a_2;
C_i(3)  = 0.9*CMb + 100*a_3;  % CTo
C_i(4)  = 7.1;         % pH_x
C_i(5)  = 0.14661;     % K_x
C_i(6)  = 1.0e-3;      % Mg_x 
C_i(7)  = 1.8e-3;      % NADH_x
C_i(8)  = 0.5e-3;      % Q
C_i(9)  = 2.6e-3;      % Cox
C_i(10) = 0.83428e-3;  % ATP_x
C_i(11) = 9.1657e-3;   % ADP_x
C_i(12) = 0.81778e-3;  % ATP_mx
C_i(13) = 7.0963e-3;   % ADP_mx
C_i(14) = 0.021511e-3; % Pi_x
C_i(15) = 6.4882e-3;   % ATP_i
C_i(16) = 0.11096e-3;  % ADP_i
C_i(17) = 0.8219e-6;   % AMP_i
C_i(18) = 6.3380e-3;   % ATP_mi
C_i(19) = 0.082636e-3; % ADP_mi
C_i(20) = 0.024551e-3; % Pi_i
C_i(21) = 1.0e-3;      % Mg_i
C_i(22) = 6.4879e-3;   % ATP_c
C_i(23) = 0.11126e-3;  % ADP_c
C_i(24) = 0.82631e-6;  % AMP_c
C_i(25) = 6.3378e-3;   % ATP_mc
C_i(26) = 0.08287e-3;  % ADP_mc
C_i(27) = 0.02350e-3;  % Pi_c
C_i(28) = 1.0e-3;      % Mg_c
C_i(29) = 12.31e-3;    % PCr_c
C_i(30) = 175.0;       % dPsi

%%
% Step 3. Putting state variables into a 30xNseg matrix for spatially
% distributed model
Co = C_i'*ones(1,Nseg);

%%
% Step 4. Verifying value of TPP
TPP = (0.472*W_c*(2*C_i(22)+C_i(23)+C_i(27)+C_i(29))*1e3 + ...
0.2*W_i*(2*C_i(15)+C_i(16)+C_i(20))*1e3 + ...
0.2*W_x*(C_i(10)+C_i(14))*1e3)/0.694

%%
% Step 5. Setting up and running a simulation
flow   = 0.75;          % (ml per min per gram of tissue);
length = 550;           % length of capillary in microns
x_AtC  = 0.45e-4;       % ATP consumption rate in mmol (s liter cell)^{-1}
FM = (rho*flow)/60;     % Mean flow in ml sec^{-1} (ml tissue)^{-1} 
vel = (FM*length)/V1;   % velocity in capillary (micron per second)
[C] = btex3_rbc(vel,length,Cin,x_AtC,C_T,P1,Nseg,Co,10);

%%
% Step 6. Plotting the predicted PO2 profiles
CtO1 = C(1,:);    % total oxygen conc in capillary
CtO2 = C(2,:);    % total oxygen conc in ISF
CtO3 = C(3,:);    % total oxygen conc in cell
PO2_1 = interp1(C_T,P1,CtO1,'linear');
PO2_2 = CtO2/a_2;
PO2_3 = (( CtO3-(C50+CMb) + ...
    sqrt( (CtO3-(C50+CMb)).^2 + 4*C50.*CtO3 ) )/2)/a_3;
x = ((1:Nseg)-0.5)./Nseg;
plot(x,PO2_1,x,PO2_2,x,PO2_3);
legend('PO_2 (cap)','PO_2 (ISF)','PO_2 (cell)');
xlabel('x/L'); ylabel('Oxygen partial pressure (mmHg)');



